/*
	Display SIR satellite data.
	
	Author: Vaughn C. Clayton
	(c) 2001 Brigham Young University
*/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <GL/glut.h>
#include <iostream>
#include <fstream>
#include <strstream>
#include <deque>
#include <sys/stat.h>
#include <errno.h>

#include "libsir.h"
#include "coord.h"
#include <GL/glui.h>

using namespace std;



#define ESCAPE 27
#define DEG2RAD (double) 0.017453292
#define RAD2DEG (double) 57.295779
#define DEG_SYMBOL ((unsigned char)176)

#define MAIN 0
#define RENDER 1
#define CONTROL 2
int window[10];

int fullscreen=0;
int x_position = 50;
int y_position = 50;
int width = 800;
int height = 600;
#define meshx 150
#define meshy 150


int xloc, yloc;
int panning=0, zooming=0, camzooming=0;
int mouse_prevx, mouse_prevy;


float *sirdata=NULL, datamin, datamax, usemin, usemax;
struct sirheader sh;
char sirfilename[1024];
char HOMEDIR[1024];

int AppWidth=800, AppHeight=600, AppX=0, AppY=0;
int fbox=0, fboy=0;
int showcursor=1;
float panscaler=1;
int ZoomEnabled=0, ZoomLocation=0, ZoomSize=100, ZoomX=50, ZoomY=50, ZoomPix=10;
float ZoomScale=2.0;
int ShowGlui=0;
float ItoR[256], ItoG[256], ItoB[256]; // Color tables
float *Palette[32]; // Color palettes
char ColorTableFile[32][256];

int CurrentPalette=0, NumPalettes=0;

GLUI *glui = NULL, *gluiStatus=NULL;
GLUI_Spinner *spinUseMax, *spinUseMin;
GLUI_StaticText *StatusBar;
GLUI_Dropdown *listPalette;




void LoadColorTable(char *fname, int ttype);
void Cleanup();

void Quit(int rcode)
{
	Cleanup();
	exit(rcode);
}

char *expandFileName(char *name)
{
	static char buf[2048];
	strcpy(buf, HOMEDIR);
	strcat(buf, name);
	return buf;
}
	

void LoadINI()
{
	FILE *file;
	char *key, *val[10], buf[2048];
	int q, ival[10];
	float fval[10];


	usemin=-314159;
	usemax=314159;
	sprintf(buf, "%s/.sir", getenv("HOME"));
	strcat(buf, "/sirview.ini");
	if((file=fopen(buf, "r"))!=NULL)
	{
		while(!feof(file))
		{
			fgets(buf, 254, file);
			
			key=strtok(buf, " =\t\r\n");
			for(q=0; q<10; q++)
			{
				val[q]=strtok(NULL, " =\t\r\n");
				fval[q]=atof(val[q]);
				ival[q]=atoi(val[q]);
			}			

			if(!strcmp(key,"WindowPos")) 
			{ 
				AppX = ival[0]; AppY = ival[1];
				AppWidth=ival[2]; AppHeight=ival[3];
			}
			if(!strcmp(key,"dataRange")) { usemin=fval[0]; usemax=fval[1]; }
			if(!strcmp(key,"ColorTable")) 
			{
				LoadColorTable(val[1], ival[0]);
				if(ival[2]!=0) CurrentPalette=NumPalettes-1;
			}
			if(!strcmp(key,"Zoom")) 
			{
				ZoomEnabled=ival[0]; 
				ZoomLocation=ival[1]; 
				ZoomScale=fval[2];
				ZoomPix=ival[3];
			}
			strncpy(buf, "\0", 254);
		}
	}
	fclose(file);
}

void SaveINI(void)
{
	FILE *f;
	int q;
	char buf[2048];

	sprintf(buf, "%s/.sir", getenv("HOME"));
	mkdir(buf, 0777);
	
	strcat(buf, "/sirview.ini");
	if((f=fopen(buf, "w"))!=NULL)
	{
		fprintf(f, "WindowPos %d %d %d %d\n", glutGet(GLUT_WINDOW_X), glutGet(GLUT_WINDOW_Y), AppWidth, AppHeight);
		fprintf(f, "dataRange %f %f\n", usemin, usemax);
		fprintf(f, "Zoom %d %d %f %d\n", ZoomEnabled, ZoomLocation, ZoomScale, ZoomPix);
		for(q=0; q<NumPalettes; q++)
			if(ColorTableFile[q][0]) fprintf(f, "ColorTable %d %s %c\n", ColorTableFile[q][0], &ColorTableFile[q][1], CurrentPalette==q?'1':' ');
	}
	fclose(f);
}


void print_errors()
{
	int error;	
	while((error=glGetError()) != GL_NO_ERROR)
		fprintf(stderr, "GL error: %s\n", gluErrorString(error));
}


void LoadColorTable(char *fname, int ttype)
{
	FILE *f;
	int q;
	unsigned char R[256], G[256], B[256];
	char buf[100], *tok;

	// Default color table is gray
	for(q=0; q<256; q++) ItoR[q]=ItoG[q]=ItoB[q]=(float)q/255.0;
	
	if(ttype==0) // None given, keep default.
	{
		ColorTableFile[NumPalettes][0]=0;
		ColorTableFile[NumPalettes][1]=0;
		return;
	}

	if(ttype==1) // Binary table, red*256, green*256, blue*256. (Not interleaved.)
	{
		f=fopen(expandFileName(fname), "rb");
		if(f==NULL)
		{
			fprintf(stderr, "Could not open file: '%s'\n", expandFileName(fname));
			return;
		}
		Palette[NumPalettes]=(float*)malloc(768*sizeof(float));
		strcpy(&ColorTableFile[NumPalettes][1], fname);
		ColorTableFile[NumPalettes][0]=1;
		fread(R, 1, 256, f);
		fread(G, 1, 256, f);
		fread(B, 1, 256, f);
		for(q=0; q<256; q++)
		{
			Palette[NumPalettes][q]=(float)R[q]/255.0;
			Palette[NumPalettes][q+256]=(float)G[q]/255.0;
			Palette[NumPalettes][q+512]=(float)B[q]/255.0;
		}
		fclose(f);
	}
	
	if(ttype==2) // ASCII table, red green blue, repeated 256 times. (Interleaved.)
	{
		f=fopen(expandFileName(fname), "rt");
		if(f==NULL)
		{
			fprintf(stderr, "Could not open file: '%s'\n", expandFileName(fname));
			return;
		}
		strcpy(&ColorTableFile[NumPalettes][1], fname);
		ColorTableFile[NumPalettes][0]=2;
		for(q=0; q<256; q++)
		{
			fgets(buf, 99, f);
			tok=strtok(buf, " =\t\r\n");
			R[q]=atoi(tok);
			tok=strtok(NULL, " =,\t\r\n");
			G[q]=atoi(tok);
			tok=strtok(NULL, " =,\t\r\n");
			B[q]=atoi(tok);

			Palette[NumPalettes][q]=(float)R[q]/255.0;
			Palette[NumPalettes][q+256]=(float)G[q]/255.0;
			Palette[NumPalettes][q+512]=(float)B[q]/255.0;
		}
		fclose(f);
	}	
	NumPalettes++;
}

void ParseOptions(int *argc, char**argv)
{
	char *key, *val[10], buf[2048];
	int q, r, ival[10];
	float fval[10];
	
	for(q=1; q<*argc; q++)
	{
		if(argv[q][0]!='-') continue; // not an option
		key=&argv[q][2];
		switch(argv[q][1])
		{
			case 'A': LoadColorTable(key, 2);
			case 'C': LoadColorTable(key, 1);
		}
		*argc--;
		for(r=q; r<*argc; r++)
			strcpy(argv[r], argv[r+1]);
	}
}


void InitGL(int Width, int Height)
{
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glClearDepth(1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();    
    glMatrixMode(GL_MODELVIEW);	
	glLoadIdentity();
}


int LoadSIRFile(char *sirname)
{
	float float_x, float_y, xs;
	long size;

	sirdata = LoadSIR(sirname, &sh);
	if(sirdata==NULL)
	{
		fprintf(stderr, "Error allocating SIR memory\n"); fflush(stderr);
	}
	print_sir_header(sh);

	float_x = -10000;
	float_y = 10000;
	for(size = sh.xpixels * sh.ypixels - 1; size>=0; size--)
	{
		xs = sirdata[size];
		if(xs < float_y) float_y = xs;
		if(xs > float_x) float_x = xs;
	}	
	datamin = float_y;
	datamax = float_x;

	float_x = usemin;
	float_y = usemax;
	usemin = float_x;
	usemax = float_y;
	
	if(usemin==-314159 && usemax==314159)
	{
		usemin = sh.v_min;
		usemax = sh.v_max;
		if(usemin<datamin) usemin=datamin;
		if(usemax>datamax) usemax=datamax;
	}

	strcpy(sirfilename, sirname);
	
	return 0;
}

void float2dms(float alpha, float &deg, float &min, float &sec)
{
	float rem;
	deg = ftrunc(alpha);
	rem = (alpha - deg) * 60;
	min = ftrunc(rem);
	sec = (rem - min) * 60;
	min = fabs(min);
	sec = fabs(sec);
}

void InitWindow()
{
	char buf[1024];
	int q;
	if(sh.xpixels<width) { width=sh.xpixels; glutReshapeWindow(width, height); }
	if(sh.ypixels<height) { height=sh.ypixels; glutReshapeWindow(width, height); }

    InitGL(width, height);

	glutSetWindow(window[MAIN]);
	sprintf(buf, "SIR View - %s", sirfilename);
	glutSetWindowTitle(buf);
	glutSetIconTitle(sirfilename);

	for(q=1; q<NumPalettes; q++)
		listPalette->add_item(q, &ColorTableFile[q][1]);


	spinUseMin->set_float_limits(datamin, datamax);
	spinUseMax->set_float_limits(datamin, datamax);
	
	if(usemin==-314159 && usemax==314159)
	{
		usemin = sh.v_min;
		usemax = sh.v_max;
		if(usemin<datamin) usemin=datamin;
		if(usemax>datamax) usemax=datamax;
	}
	
	GLUI_Master.sync_live_all();
}

void init(int argc, char **argv)
{
	int q;
	
	// Default color table is gray
	Palette[0]=(float*)malloc(768*sizeof(float));
	for(q=0; q<256; q++) Palette[0][q]=Palette[0][q+256]=Palette[0][q+512]=(float)q/255.0;
	ColorTableFile[0][0]=0;
	ColorTableFile[0][1]=0;
	NumPalettes=1;
	
	sprintf(HOMEDIR, "%s/.sir/", getenv("HOME"));

	LoadINI();
	ParseOptions(&argc, argv);

	LoadSIRFile(argv[1]);
	
	xloc = 0;//sh.xpixels / 2;
	yloc = 0;//sh.ypixels / 2;	
}


void DrawGLScene()
{
	print_errors();

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, width, 0, height, -1, 1);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// Allows use of color tables
	glPixelTransferi(GL_MAP_COLOR, GL_TRUE);
	glPixelMapfv(GL_PIXEL_MAP_R_TO_R, 256, Palette[CurrentPalette]);
	glPixelMapfv(GL_PIXEL_MAP_G_TO_G, 256, Palette[CurrentPalette]+256);
	glPixelMapfv(GL_PIXEL_MAP_B_TO_B, 256, Palette[CurrentPalette]+512);
	
	// Scale and bias allow us to limit viewable data value range
	float scale, bias;	
	if(usemax==usemin) scale=256;
	else scale = 1.0 / (usemax-usemin);
	bias = (-usemin) * scale;	
	glPixelTransferf(GL_RED_SCALE, scale);
	glPixelTransferf(GL_GREEN_SCALE, scale);
	glPixelTransferf(GL_BLUE_SCALE, scale);
	glPixelTransferf(GL_RED_BIAS, bias);
	glPixelTransferf(GL_GREEN_BIAS, bias);
	glPixelTransferf(GL_BLUE_BIAS, bias);


	glRasterPos2i(0,0);
	glPixelZoom(1.0, 1.0);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, sh.xpixels);
	glPixelStorei(GL_UNPACK_SKIP_ROWS, yloc);
	glPixelStorei(GL_UNPACK_SKIP_PIXELS, xloc);
	glDrawPixels(width, height, GL_LUMINANCE, GL_FLOAT, sirdata);
	
	// Zooming Crosshair
	if(zooming)
	{
        glEnable( GL_LINE_STIPPLE );
        glLineStipple( 1, 0x5555 );
		glBegin(GL_LINES);
		glVertex2i(ZoomX-xloc, 0); glVertex2i(ZoomX-xloc, height);
		glVertex2i(0, ZoomY-yloc); glVertex2i(width, ZoomY-yloc);
		glEnd();
        glDisable( GL_LINE_STIPPLE );
	}

	ZoomSize = ZoomPix * ZoomScale + 0.1;
	if(ZoomEnabled)
	{
		int px;
		px = ZoomSize / ZoomScale / 2;
		int z1, z2, z3, z4;
		int cx, cy;

		glColor3f( 1.0, 0.0, 0.0 );

		glBegin(GL_LINE_LOOP);
		glVertex2i(ZoomX-px-xloc, ZoomY-px-yloc);
		glVertex2i(ZoomX+px-xloc, ZoomY-px-yloc);
		glVertex2i(ZoomX+px-xloc, ZoomY+px-yloc);
		glVertex2i(ZoomX-px-xloc, ZoomY+px-yloc);
		glEnd();
		
		switch(ZoomLocation)
		{
			case 0:
				z1 = ZoomSize;
				z2 = height;
				z3 = height-ZoomSize-1;
				z4 = 0;
				cx = ZoomSize/2; cy = height-(ZoomSize/2);
				glRasterPos2i(0, height-ZoomSize); 
				glScissor(fbox, height-ZoomSize+fboy, ZoomSize, ZoomSize);
				break;
			case 1: 
				z1 = width-ZoomSize-1;
				z2 = height;
				z3 = height-ZoomSize-1;
				z4 = width;
				cx = width-(ZoomSize/2); cy = height-(ZoomSize/2);
				glRasterPos2i(width-ZoomSize, height-ZoomSize); 
				glScissor(width+fbox-ZoomSize, height-ZoomSize+fboy, ZoomSize, ZoomSize);
				break;
			case 2:
				z1 = ZoomSize;
				z2 = 0;
				z3 = ZoomSize;
				z4 = 0;
				cx = ZoomSize/2; cy = ZoomSize/2;
				glRasterPos2i(0, 0);
				glScissor(fbox, fboy, ZoomSize, ZoomSize);
				break;
			case 3:
				z1 = width-ZoomSize-1;
				z2 = 0;
				z3 = ZoomSize;
				z4 = width;
				cx = width-ZoomSize/2; cy = ZoomSize/2;
				glRasterPos2i(width-ZoomSize, 0);
				glScissor(width+fbox-ZoomSize, fboy, ZoomSize, ZoomSize);
				break;
		}

		glBegin(GL_LINE_STRIP);
		glVertex2i(z1, z2); 
		glVertex2i(z1, z3);
		glVertex2i(z4, z3);
		glEnd();				

		glEnable(GL_SCISSOR_TEST);
		glPixelZoom(ZoomScale, ZoomScale);
		
		glPixelStorei(GL_UNPACK_ROW_LENGTH, sh.xpixels);
		glPixelStorei(GL_UNPACK_SKIP_ROWS, ZoomY-px);
		glPixelStorei(GL_UNPACK_SKIP_PIXELS, ZoomX-px);
		px = ZoomSize / ZoomScale + ZoomScale;
		glDrawPixels(px, px, GL_LUMINANCE, GL_FLOAT, sirdata);
		glDisable(GL_SCISSOR_TEST);
		
		cx += (ZoomPix & 1) ? 0: ZoomScale/2;
		cy += (ZoomPix & 1) ? 0: ZoomScale/2;
		glBegin(GL_LINE_LOOP);
		glVertex2i(cx-ZoomScale/2, cy-ZoomScale/2); 
		glVertex2i(cx+ZoomScale/2, cy-ZoomScale/2);
		glVertex2i(cx+ZoomScale/2, cy+ZoomScale/2);
		glVertex2i(cx-ZoomScale/2, cy+ZoomScale/2);
		glEnd();				
		
	}

	print_errors();
    glutSwapBuffers();
}




void ReSizeGLScene(int Width, int Height)
{
	int tx, ty, tw, th;

    if (Height<=0) //Prevent A Divide By Zero If The Window Is Too Small
		Height=1;
	if(Width<=0)
		Width=1;
	AppWidth = Width;
	AppHeight = Height;

	width=Width;
	height=Height;
	glutReshapeWindow(Width, Height);

	tx = 0;
	ty = 0;
	tw = width;
	th = height;
	fbox=0;
	fboy=0;

	GLUI_Master.get_viewport_area(&tx, &ty, &tw, &th);
	width = tw;
	height = th;
	fbox = tx;
	fboy = ty;
		
    glViewport(tx, ty, tw, th);
}





void keyPressed(unsigned char key, int x, int y) 
{
	
	switch (key) 
	{
	    case 27:	// ESC
			Quit(0);
		case 'L': case 'l':
			break;
		case 'z':case 'Z': ZoomEnabled=!ZoomEnabled; break;
		case '=': ZoomSize++; break;
		case '-': ZoomSize--; break;
	}
	glutPostRedisplay();
}

void specialKeyPressed(int key, int x, int y) 
{
	switch (key) 
	{
		case GLUT_KEY_F1:
			fullscreen = !fullscreen;
			if (fullscreen) 
			{
				glutSetWindow(window[MAIN]);
				x_position = glutGet((GLenum)GLUT_WINDOW_X);	/* Save parameters */
				y_position = glutGet((GLenum)GLUT_WINDOW_Y);
				width = glutGet((GLenum)GLUT_WINDOW_WIDTH);
				height = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
				glutFullScreen();				/* Go to full screen */
			}
			else
			{
				glutSetWindow(window[MAIN]);
				glutReshapeWindow(width, height);		/* Restore us */
				glutPositionWindow(x_position, y_position);
			}
			break;			
		case GLUT_KEY_UP:
			break;
		case GLUT_KEY_DOWN:
			break;
	}

	glutPostRedisplay();
}


void MouseMotion(int x, int y);
void MouseButton(int button, int state, int x, int y)
{
	static int buttons=0;
	
	mouse_prevx = x;
	mouse_prevy = y;

	switch(button)
	{
		case GLUT_LEFT_BUTTON: buttons = (state==GLUT_UP? buttons&(~1) : buttons|1); break;
		case GLUT_MIDDLE_BUTTON: buttons = (state==GLUT_UP? buttons&(~2) : buttons|2); break;
		case GLUT_RIGHT_BUTTON: buttons = (state==GLUT_UP? buttons&(~4) : buttons|4); break;
	}

	state = glutGetModifiers();
	switch(state)
	{
		case GLUT_ACTIVE_SHIFT: buttons ^= 8; break;
		case GLUT_ACTIVE_CTRL: buttons ^= 16; break;
		case GLUT_ACTIVE_ALT: buttons ^= 32; break;
	}

	panning=0;
	zooming=0;
	camzooming=0;
	switch(buttons)
	{
		case 8: 
		case 16: 
		case 32:
		case 0: // All buttons released
			if(!showcursor) glutSetCursor(GLUT_CURSOR_NONE);
			else glutSetCursor(GLUT_CURSOR_INHERIT);
			break;
		case 1: // Left
			zooming=1;
			if(!showcursor) glutSetCursor(GLUT_CURSOR_NONE);
			else glutSetCursor(GLUT_CURSOR_NONE);
			GLUI_Master.sync_live_all();
			MouseMotion(x, y);
			break;
		case 2: // Middle
			panning=1;
			if(!showcursor) glutSetCursor(GLUT_CURSOR_NONE);
			else glutSetCursor(GLUT_CURSOR_CROSSHAIR);
			break;
		case 3: // Left + Middle
		case 4: //Right
			if(ShowGlui) glui->hide();
			else glui->show();
			ShowGlui=!ShowGlui;
			ReSizeGLScene(AppWidth, AppHeight);
			break;
		case 9: // Shift + Left + Middle
			break;
	}
	glutPostRedisplay();
}


void MouseMotion(int x, int y)
{
	float m, fx, fy;
	char buf[255];
	int px, py;
	
	px = xloc+x-fbox;
	py = yloc+(height-y);
	

	if(zooming)
	{
		int zsh = ZoomSize / 2, zshh=zsh/2;
		ZoomX = px;
		ZoomY = py;

		pixtolatlon(sh, px, py, &fx, &fy);
		if(px>=0 && py>=0 && px<sh.xpixels && py<sh.ypixels) 
			m=sirdata[px+py*sh.xpixels];
		else m=0;

		float d1, m1, s1, d2, m2, s2;
		float2dms(fx, d1, m1, s1);
		float2dms(fy, d2, m2, s2);
		sprintf(buf, "X,Y: %d,%d   Lon,Lat: %3.3f,%3.3f  (%.0f%c%.0f'%.3f\", %.0f%c%.0f'%.3f\")  Value: %f", 
			px,py,fx,fy,d1,DEG_SYMBOL,m1,s1,d2,DEG_SYMBOL,m2,s2,m);
		StatusBar->set_text(buf);	
		
		if(ZoomX<zshh) ZoomX = zshh;
		if(ZoomX>sh.xpixels-zshh) ZoomX = sh.xpixels-zshh;
		if(ZoomY<zshh) ZoomY = zshh;
		if(ZoomY>sh.ypixels-zshh) ZoomY = sh.ypixels-zshh;
	}
	else if(panning)
	{
		xloc -= (x-mouse_prevx);
		yloc += (y-mouse_prevy);
		if(xloc<0) xloc=0;
		if(yloc<0) yloc=0;
		if(xloc+width > sh.xpixels) xloc=sh.xpixels-width;
		if(yloc+height > sh.ypixels) yloc=sh.ypixels-height;
	}
	else if(camzooming)
	{
	}
	
	mouse_prevx = x;
	mouse_prevy = y;
	glutPostRedisplay();
}



void Cleanup()
{
	int q;
	SaveINI();
	free(sirdata);
	for(q=0; q<NumPalettes; q++) free(Palette[q]);
}




int InitUI()
{
	GLUI_Spinner *spin;
	GLUI_Rollout *roll;
	GLUI_Dropdown *list;

	glui = GLUI_Master.create_glui_subwindow(window[MAIN], GLUI_SUBWINDOW_LEFT);
	glui->set_main_gfx_window(window[MAIN]);
	glui->hide();
	
	roll = glui->add_rollout("Zoom");
	glui->add_checkbox_to_panel(roll, "Enable", &ZoomEnabled);
	spin = glui->add_spinner_to_panel(roll, "Scale", GLUI_SPINNER_FLOAT, &ZoomScale);
	spin->set_float_limits(1.0, 10.0);
	
	spin = glui->add_spinner_to_panel(roll, "Size", GLUI_SPINNER_INT, &ZoomPix);
	spin->set_int_limits(5, 100);
	
	GLUI_RadioGroup *ScaleRadio = 
		glui->add_radiogroup_to_panel(roll, &ZoomLocation);
	glui->add_radiobutton_to_group(ScaleRadio, "Top Left");
	glui->add_radiobutton_to_group(ScaleRadio, "Top Right");
	glui->add_radiobutton_to_group(ScaleRadio, "Bottom Left");
	glui->add_radiobutton_to_group(ScaleRadio, "Bottom Right");
	roll->close();
	
//	glui->add_column(false);
	
	roll = glui->add_rollout("Colors");
	spinUseMin = glui->add_spinner_to_panel(roll,"Min", GLUI_SPINNER_FLOAT, &usemin);
	spinUseMax = glui->add_spinner_to_panel(roll,"Max", GLUI_SPINNER_FLOAT, &usemax);
//	list = glui->add_picklist_to_panel(roll, "Palette", &CurrentPalette);
	list = glui->add_dropdown_to_panel(roll, "Palette", &CurrentPalette);
	list->add_item(0, "Grayscale");
	listPalette=list;
	roll->close();


	gluiStatus = GLUI_Master.create_glui_subwindow(window[MAIN], GLUI_SUBWINDOW_BOTTOM);
	StatusBar = gluiStatus->add_statictext("sirview version 1.1");

	return 0;
}

int main(int argc, char **argv) 
{
	int q;
    glutInit(&argc, argv);
	init(argc, argv);

    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE  | GLUT_DEPTH);  
    glutInitWindowSize(AppWidth, AppHeight);
    glutInitWindowPosition(AppX, AppY);
    window[MAIN] = glutCreateWindow("SIR View");
	glutMotionFunc(MouseMotion);	
    glutDisplayFunc(DrawGLScene);  
	
	InitUI();
    GLUI_Master.set_glutKeyboardFunc(keyPressed);
    GLUI_Master.set_glutSpecialFunc(specialKeyPressed);
	GLUI_Master.set_glutMouseFunc(MouseButton);
	GLUI_Master.set_glutReshapeFunc(ReSizeGLScene);
	GLUI_Master.set_glutIdleFunc(NULL);
	GLUI_Master.sync_live_all();

	glutSetWindow(window[MAIN]);
	InitWindow();
	
    glutMainLoop();  

    return 0;
}

